@@ -6,15 +6,14 @@
 
 #pragma once
 
-#include "xdgshell_interface.h"
 #include "qwayland-server-xdg-shell.h"
+#include "xdgshell_interface.h"
 
 #include "surface_interface.h"
 #include "surfacerole_p.h"
 
 namespace KWaylandServer
 {
-
 class XdgToplevelDecorationV1Interface;
 
 class XdgShellInterfacePrivate : public QtWaylandServer::xdg_wm_base
@@ -84,6 +83,13 @@ protected:
     void xdg_positioner_set_parent_configure(Resource *resource, uint32_t serial) override;
 };
 
+struct XdgSurfaceState {
+    QRect windowGeometry;
+    quint32 acknowledgedConfigure;
+    bool acknowledgedConfigureIsSet = false;
+    bool windowGeometryIsSet = false;
+};
+
 class XdgSurfaceInterfacePrivate : public QtWaylandServer::xdg_surface
 {
 public:
@@ -100,13 +106,8 @@ public:
     bool isMapped = false;
     bool isConfigured = false;
 
-    struct State
-    {
-        QRect windowGeometry;
-    };
-
-    State next;
-    State current;
+    XdgSurfaceState next;
+    XdgSurfaceState current;
 
     static XdgSurfaceInterfacePrivate *get(XdgSurfaceInterface *surface);
 
@@ -138,8 +139,7 @@ public:
     QString windowTitle;
     QString windowClass;
 
-    struct State
-    {
+    struct State {
         QSize minimumSize;
         QSize maximumSize;
     };
